package hiro.yoshioka.sdh;

import hiro.yoshioka.util.StringUtil;

import java.io.Serializable;
import java.sql.ResultSetMetaData;
import java.sql.SQLException;
import java.sql.Types;
import java.util.logging.Level;
import java.util.logging.Logger;

public class ResultSetMetaCopy implements ResultSetMetaData, Serializable {
	int columnCount;
	int[] columnDisplaySize;
	String[] columnName;
	int[] columnType;
	String[] columnTypeName;
	int[] precision;
	int[] scale;
	String[] schemaName;
	String[] tableName;

	public ResultSetMetaCopy(ResultSetMetaData meta) {
		try {
			columnCount = meta.getColumnCount();
			columnDisplaySize = new int[columnCount];
			columnName = new String[columnCount];
			columnType = new int[columnCount];
			columnTypeName = new String[columnCount];
			precision = new int[columnCount];
			scale = new int[columnCount];
			schemaName = new String[columnCount];
			tableName = new String[columnCount];
			for (int i = 1; i <= columnCount; i++) {
				columnDisplaySize[i - 1] = meta.getColumnDisplaySize(i);
				columnName[i - 1] = meta.getColumnName(i);
				columnType[i - 1] = meta.getColumnType(i);
				columnTypeName[i - 1] = meta.getColumnTypeName(i);
				try {
					precision[i - 1] = meta.getPrecision(i);
				} catch (Throwable e) {
					precision[i - 1] = 0;
				}
				try {
					scale[i - 1] = meta.getScale(i);
				} catch (Throwable e) {
					scale[i - 1] = 0;
				}
				try {
					schemaName[i - 1] = meta.getSchemaName(i);
					tableName[i - 1] = meta.getTableName(i);
				} catch (Throwable e) {
					Logger.getLogger(getClass().getName()).info(
							columnName[i - 1]
									+ " cant get SchemaName/TableName ");
				}
			}
		} catch (Throwable e) {
			Logger.getLogger(getClass().getName()).log(Level.SEVERE, "", e);
		}
	}

	public void resetTableName(String name) {
		for (int i = 0; i < tableName.length; i++) {
			if (StringUtil.isEmpty(tableName[i])) {
				tableName[i] = name;
			}
		}
	}

	public String getCatalogName(int column) throws SQLException {
		return null;
	}

	public String getColumnClassName(int column) throws SQLException {
		return null;
	}

	public int getColumnCount() throws SQLException {
		return columnCount;
	}

	public int getColumnDisplaySize(int column) throws SQLException {
		return columnDisplaySize[column - 1];
	}

	public String getColumnLabel(int column) throws SQLException {
		return null;
	}

	public String getColumnName(int column) throws SQLException {
		return columnName[column - 1];
	}

	public int getColumnType(int column) throws SQLException {
		return columnType[column - 1];
	}

	public String getColumnTypeName(int column) throws SQLException {
		return columnTypeName[column - 1];
	}

	public int getPrecision(int column) throws SQLException {
		return precision[column - 1];
	}

	public int getScale(int column) throws SQLException {
		return scale[column - 1];
	}

	public String getSchemaName(int column) throws SQLException {
		return schemaName[column - 1];
	}

	public String getTableName(int column) throws SQLException {
		return tableName[column - 1];
	}

	public boolean isAutoIncrement(int column) throws SQLException {
		return false;
	}

	public boolean isCaseSensitive(int column) throws SQLException {
		return false;
	}

	public boolean isCurrency(int column) throws SQLException {
		return false;
	}

	public boolean isDefinitelyWritable(int column) throws SQLException {
		return false;
	}

	public int isNullable(int column) throws SQLException {
		return 0;
	}

	public boolean isReadOnly(int column) throws SQLException {
		return false;
	}

	public boolean isSearchable(int column) throws SQLException {
		return false;
	}

	public boolean isSigned(int column) throws SQLException {
		return false;
	}

	public boolean isWritable(int column) throws SQLException {
		return false;
	}

	public boolean isWrapperFor(Class<?> arg0) throws SQLException {
		return false;
	}

	public <T> T unwrap(Class<T> arg0) throws SQLException {
		return null;
	}

	public String getBlobOrBinary() {
		StringBuilder buf = new StringBuilder();
		for (int i = 1; i <= columnCount; i++) {
			if (isBlob(i) || isBinaly(i)) {
				if (buf.length() > 0) {
					buf.append(",");
				}
				buf.append(columnName[i - 1]);
			}
		}
		return buf.toString();
	}

	public boolean hasBlobOrBinary() {
		for (int i = 1; i <= columnCount; i++) {
			if (isBlob(i) || isBinaly(i)) {
				return true;
			}
		}
		return false;
	}

	public boolean isBlob(int i) {
		if (i <= 0) {
			return false;
		}
		return columnType[i - 1] == Types.BLOB;
	}

	public boolean isBinaly(int i) {
		if (i <= 0) {
			return false;
		}
		int type = columnType[i - 1];
		if (type == Types.BINARY || type == Types.VARBINARY
				|| type == Types.LONGVARBINARY) {
			return true;
		}
		return false;
	}

	public boolean needsQuote(int index) {
		if (isTypeString(index) || isTypeTimes(index)) {
			return true;
		}
		return false;
	}

	public boolean isTypeTimes(int index) {
		switch (columnType[index - 1]) {
		case Types.TIMESTAMP:
			return true;
		case Types.DATE:
			return true;
		case Types.TIME:
			return true;
		}
		return false;
	}

	public boolean isTypeString(int i) {
		switch (columnType[i - 1]) {
		case Types.VARCHAR:
			return true;
		case Types.CHAR:
			return true;
		case Types.LONGNVARCHAR:
			return true;
		case Types.LONGVARCHAR:
			return true;
		case Types.NCHAR:
			return true;
		case Types.NVARCHAR:
			return true;
		default:
			break;
		}
		return false;
	}

	public int[] getColumnDisplaySize() {
		return columnDisplaySize;
	}

	public void setColumnDisplaySize(int[] columnDisplaySize) {
		this.columnDisplaySize = columnDisplaySize;
	}

	public String[] getColumnName() {
		return columnName;
	}

	public void setColumnName(String[] columnName) {
		this.columnName = columnName;
	}

	public int[] getColumnType() {
		return columnType;
	}

	public void setColumnType(int[] columnType) {
		this.columnType = columnType;
	}

	public String[] getColumnTypeName() {
		return columnTypeName;
	}

	public void setColumnTypeName(String[] columnTypeName) {
		this.columnTypeName = columnTypeName;
	}

	public int[] getPrecision() {
		return precision;
	}

	public void setPrecision(int[] precision) {
		this.precision = precision;
	}

	public int[] getScale() {
		return scale;
	}

	public void setScale(int[] scale) {
		this.scale = scale;
	}

	public String[] getSchemaName() {
		return schemaName;
	}

	public void setSchemaName(String[] schemaName) {
		this.schemaName = schemaName;
	}

	public String[] getTableName() {
		return tableName;
	}

	public void setTableName(String[] tableName) {
		this.tableName = tableName;
	}

	public void setColumnCount(int columnCount) {
		this.columnCount = columnCount;
	}

}
